from ..imports import datetime,List, Dict
from ..query_utils import *

class UserIPManager(BaseQueryManager):
    """Manages CRUD on uploads; all your original methods just wired up dynamically."""
    def __init__(self, logs_on: bool = False):
        self.filename = 'userIpQueries'
        self.basename = f'{self.filename}.json'
        self.logs_on = False
        super().__init__(self.filename, logs_on=logs_on)


    def select_user_ip(
        self,
        user_id: int,
        ip: str
        ):
        """Check if a user IP record exists in the user_ips table."""
        if self.logs_on:
            initialize_call_log()
        query = self._query_select_user_ip
        args = (user_id, ip)
        return select_rows(query, *args)


    def update_user_ip(
        self,
        user_id: int,
        ip: str
        ):
        """Update the last_seen timestamp and increment hit_count for a user IP."""
        if self.logs_on:
            initialize_call_log()
        query = self._query_update_user_ip
        time_now = datetime.utcnow()
        args = (time_now, user_id, ip)
        insert_query(query, *args)


    def insert_user_ip(
        self,
        user_id: int,
        ip: str
        ):
        """Insert a new user IP record into the user_ips table."""
        if self.logs_on:
            initialize_call_log()
        query = self._query_insert_user_ip
        args = (user_id, ip)
        insert_query(query, *args)


    def log_user_ip(
        self,
        user_id: int,
        ip: str
        ):
        """
        Insert or update the user_ips table for the given (user_id, ip) pair.
        If the record exists, update it; otherwise, insert a new record.
        """
        if self.logs_on:
            initialize_call_log()
       
        rows = self.select_user_ip(user_id, ip)
        if rows:
            self.update_user_ip(user_id, ip)
        else:
            self.insert_user_ip(user_id, ip)


    def select_user_by_ip(
        self,
        ip: str
        ):
        """Retrieve user details associated with the given IP address."""
        if self.logs_on:
            initialize_call_log()
        query = self._query_select_user_by_ip
        args = (ip,)
        return select_rows(query, *args)


    def get_user_by_ip(
        self,
        ip: str
        ):
        """
        Return all users who have been seen from the given IP, along with timestamps.
        Initializes call logging before querying.
        """
        if self.logs_on:
            initialize_call_log()
      
        return self.select_user_by_ip(ip)



from py import *
from abvstract_database import (
    connectionManager,
    connect_db,
    get_env_value,
    RealDictCursor,
    psycopg2,
    logging,
    warnings,
    traceback,
    sql,make_list,
    SingletonMeta,
    initialize_call_log,
    logging,
    sql,
    connect,
    make_list,
    SingletonMeta
    )


def get_splits(string,splits=[],direction=0):
    for split in splits:
        nustring = string.split(split)
        string =  nustring[0]
        if len(nustring)> direction:
            string = nustring[direction]
    return string
def get_db_row_name(query):
    splits = ['FROM ','UPDATE ','CREATE TABLE IF NOT EXISTS ','INSERT INTO ']
    query = get_splits(query,splits=splits,direction=1)
    splits = [' ','\t','\n']
    row_name = get_splits(query,splits=splits,direction=0)
    return row_name
def get_kargs(*args,**kwargs):
    return kwargs
def get_row_keys(query):
    row_name = get_db_row_name(query)
    query = f"SELECT * FROM {row_name} ORDER BY RANDOM() LIMIT 1;"
    row = execute_query(query)
    keys = row[0].keys()
    return keys
def return_only_db_inputs(query,**kwargs):
    return_kwargs = {}
    input_keys = get_row_keys(query)
    for input_key in input_keys:
        if input_key in kwargs:
            return_kwargs[input_key] = kwargs.get(input_key)
    return return_kwargs
def get_query_result(query, conn):
    """Executes a query and returns the results or commits the transaction."""
    with conn.cursor() as cursor:
        cursor.execute(query)
        if query.strip().lower().startswith("select"):
            return cursor.fetchall()  # Return data for SELECT queries
        conn.commit()

def query_data(query, values=None, error="Error executing query:", zipRows=True):
    """Execute a query and handle transactions with error management."""
    with get_connection() as conn:
        # Choose the cursor type based on whether you want to zip rows with column names
        cursor_factory = DictCursor if zipRows else None
        with conn.cursor(cursor_factory=cursor_factory) as cursor:
            try:
                cursor.execute(query, values)
                result = cursor.fetchall()
                # Log the first row to see its structure
                if result:
                    logging.info("First row data structure: %s", result[0])
                return result
            except Exception as e:
                conn.rollback()
                logging.error("%s %s\nValues: %s\n%s", error, query, values, e)

#####################################
# Fix #1: Correct the usage in fetch_any_combo
#####################################
def fetch_any_combo(columnNames='*',
                    tableName=None,
                    searchColumn=None,
                    searchValue=None,
                    anyValue=False,
                    zipIt=True,
                    schema='public'):
    """
    Fetch rows based on dynamic SQL built from parameters.
    
    :param columnNames: Comma separated columns or '*' for all.
    :param tableName: The table to query. Must not be None or '*'.
    :param searchColumn: The column on which to filter.
    :param searchValue: The value to match in searchColumn.
    :param anyValue: If True, uses = ANY(%s) for arrays.
    :param zipit: If True, uses DictCursor in query_data.
    :param schema: The DB schema.
    """
    if not tableName or tableName == '*':
        logging.error("Invalid tableName provided to fetch_any_combo: %s", tableName)
        return []  # or raise an Exception

    # Build the SELECT list
    if columnNames == '*':
        select_cols = sql.SQL('*')
    else:
        # Convert "col1,col2" -> [col1, col2]
        col_list = [c.strip() for c in columnNames.split(',')]
        select_cols = sql.SQL(", ").join(sql.Identifier(col) for col in col_list)

    # Build the base query: SELECT ... FROM schema.tableName
    base_query = sql.SQL("SELECT {} FROM {}.{}").format(
        select_cols,
        sql.Identifier(schema),
        sql.Identifier(tableName)
    )

    # Build the WHERE clause if needed
    params = []
    if searchColumn and searchValue is not None:
        if anyValue:
            base_query += sql.SQL(" WHERE {} = ANY(%s)").format(sql.Identifier(searchColumn))
            params.append(searchValue if isinstance(searchValue, list) else [searchValue])
        else:
            base_query += sql.SQL(" WHERE {} = %s").format(sql.Identifier(searchColumn))
            params.append(searchValue)

    
    if zipIt:
        result = query_data_as_dict(base_query, values=params)
    else:
        result = query_data(base_query, values=params, zipRows=zipit)
    return result
def get_anything(*args, **kwargs):
    if args:
        for arg in args:
            if 'tableName' not in kwargs:
                kwargs['tableName'] = arg
    response = fetch_any_combo(**kwargs)
    logging.info("Received data: %s", response)  # Log to see the data
    if isinstance(response, list) and len(response) == 1:
        response = response[0]
    return response

def get_table_name_from_query(query):
    """Extract table name from SQL query."""
    if isinstance(query, sql.Composed):
        query = query.as_string(get_connection())  # Convert to string
    parts = query.split()
    if 'from' in parts:
        return parts[parts.index('from') + 1]
    return None
class columnNamesManager(metaclass=SingletonMeta):
    def __init__(self):
        if not hasattr(self, 'initialized'):
            self.initialized = True
            self.columnNames = {}

    def get_column_names(self, tableName, schema='public'):
        if tableName not in self.columnNames:
            self.columnNames[tableName] = self.fetch_column_names(tableName, schema)
        return self.columnNames[tableName]

    def fetch_column_names(self, tableName, schema='public'):
        query = """
            SELECT column_name
            FROM information_schema.columns
            WHERE table_name = %s AND table_schema = %s
            ORDER BY ordinal_position;
        """
        results = query_data(query, [tableName, schema], error='Error fetching column names')
        if not results:
            logging.warning(f"No columns found for table {tableName} in schema {schema}")
            return []
        return [row[0] for row in results]

    def zip_rows(self, tableName, rows, schema='public'):
        column_names = self.get_column_names(tableName, schema)
        return [dict(zip(column_names, make_list(row))) for row in rows]



from ..imports import datetime,List, Dict
from ..query_utils import *

class UserManager(BaseQueryManager):
    """Manages CRUD on uploads; all your original methods just wired up dynamically."""
    def __init__(self, logs_on: bool = True):
        self.filename = 'userQueries'
        self.basename = f'{self.filename}.json'
     
        super().__init__(self.filename, logs_on=logs_on)



    def get_existing_users(
        self
        ):
        """Check if a user IP record exists in the user_ips table."""
        if self.logs_on:
            initialize_call_log()
        query = self._query_select_existing_user 
        rows = select_rows(query) or []
  
        return get_rows(rows)


    def get_insert_update_user(
        self,username: str,
        plaintext_pwd: str,
        is_admin: bool = None
        ) -> None:
        """Update the last_seen timestamp and increment hit_count for a user IP."""
        if self.logs_on:
            initialize_call_log()
        is_admin = is_admin or False
        hashed = bcrypt_plain_text(plaintext_pwd,rounds=12)
        query = self._query_insert_update_user
        args = (username, hashed, is_admin,)
        insert_query(query,*args)

    def get_select_user_by_username(
        self,
        username: str
        ) -> None:
        """Insert a new user IP record into the user_ips table."""
        if self.logs_on:
            initialize_call_log()
        query = self._query_select_user_by_username
        args = (username,)
        rows = select_rows(query,*args)
        return get_rows(rows)




from ..imports import datetime,List, Dict
from ..query_utils import *

class UploadManager(BaseQueryManager):
    """
    Manager for 'uploads' table: insert, select, update share settings.
    """
    def __init__(self, logs_on: bool = True):
        super().__init__('UploadsQueries', logs_on=logs_on)

    def insert_upload_items(self, **kwargs) -> Dict[str, Any]:
        allowed = {
            'filename', 'filepath', 'uploader_id',
            'shareable', 'download_count',
            'download_limit', 'share_password'
        }
        data = {k: v for k, v in kwargs.items() if k in allowed}
        return self.insert('uploads', data)

    def select_upload_from_id(self, upload_id: int) -> Optional[Dict[str, Any]]:
        return self.run('select_upload', upload_id, one=True)

    def update_upload(self, upload_id: int, **kwargs) -> Dict[str, Any]:
        """
        Update the uploads row with given kwargs for the matching upload_id.
        """
        allowed = {
            'filename', 'filepath', 'uploader_id',
            'shareable', 'download_count',
            'download_limit', 'share_password'
        }
        data = {k: v for k, v in kwargs.items() if k in allowed}
        where = {'id': upload_id}
        return self.update('uploads', data, where)

    def insert_upload_items(
        self,
        filename: str,
        filepath: str,
        uploader_id: str = None,
        shareable: bool = False,
        download_count: int = 0,
        download_limit: int = None,
        share_password: bool = False,
    ) -> int:
        if self.logs_on:
            initialize_call_log()
        query = self._query_insert_upload_items
        args = (
            filename,
            filepath,
            uploader_id,
            shareable,
            download_count,
            download_limit,
            share_password,
        )
        row = select_rows(query, *args)
        if row and "id" in row:
            return row["id"]
        raise ValueError("Failed to create fileId: no ID returned from database")

    def select_upload_from_id(self, file_id: int) -> Dict:
        if self.logs_on:
            initialize_call_log()
        query = self._query_select_upload_from_id
        args = (file_id,)
        row = select_rows(query, *args)
        return get_rows(row)

    def update_upload_share_link(
        self,
        shareable: bool,
        pwd_plain: str,
        download_limit: int,
        file_id: int,
    ) -> None:
        if self.logs_on:
            initialize_call_log()
        query = self._query_update_upload_share_link
        args =  (shareable, pwd_plain, download_limit, file_id,)
        execute_query(query,*args)

    def select_upload_from_filepath(self, filepath: str) -> Dict:
        if self.logs_on:
            initialize_call_log()
        query = self._query_select_upload_from_filepath
        args = (file_id,)
        row = select_rows(query, *args)
        return get_rows(row)

    def get_by_user(self, uploader_id: int) -> List[Dict]:
        return select_rows(self._query_select_upload_items, uploader_id)

    def create(self, filename: str, filepath: str, **kwargs) -> int:
        # assumes JSON has 'query_insert_upload_items_returning'
        return select_rows(self._query_insert_upload_items, filename, filepath, **tuple(kwargs.values()))

    def update_share(self, file_id: int, **kwargs) -> None:
        execute_query(self._query_update_upload_share_link, file_id, **tuple(kwargs.values()))

    def get_one(self, file_id: int) -> Optional[Dict]:
        return select_rows(self._query_select_upload_from_id, file_id)
